@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2010 Jeffrey Stedfast
+ *  Copyright (C) 2000-2011 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -42,6 +42,8 @@ typedef struct _GMimeCryptoContext GMimeCryptoContext;
 typedef struct _GMimeCryptoContextClass GMimeCryptoContextClass;
 
 typedef struct _GMimeSigner GMimeSigner;
+typedef struct _GMimeCryptoRecipient GMimeCryptoRecipient;
+typedef struct _GMimeDecryptionResult GMimeDecryptionResult;
 typedef struct _GMimeSignatureValidity GMimeSignatureValidity;
 
 
@@ -63,60 +65,36 @@ typedef gboolean (* GMimePasswordRequestFunc) (GMimeCryptoContext *ctx, const ch
 					       gboolean reprompt, GMimeStream *response, GError **err);
 
 
-
-/**
- * GMimeCryptoPubKeyAlgo:
- * @GMIME_CRYPTO_PUBKEY_ALGO_DEFAULT: The default public-key algorithm.
- * @GMIME_CRYPTO_PUBKEY_ALGO_RSA: The RSA algorithm.
- * @GMIME_CRYPTO_PUBKEY_ALGO_RSA_E: An encryption-only RSA algorithm.
- * @GMIME_CRYPTO_PUBKEY_ALGO_RSA_S: A signature-only RSA algorithm.
- * @GMIME_CRYPTO_PUBKEY_ALGO_ELG: The ElGamal algorithm.
- * @GMIME_CRYPTO_PUBKEY_ALGO_ELG_E: An encryption-only ElGamal algorithm.
- * @GMIME_CRYPTO_PUBKEY_ALGO_DSA: The DSA algorithm.
- *
- * A public-key algorithm.
- **/
-typedef enum {
-	GMIME_CRYPTO_PUBKEY_ALGO_DEFAULT,
-	GMIME_CRYPTO_PUBKEY_ALGO_RSA,
-	GMIME_CRYPTO_PUBKEY_ALGO_RSA_E,
-	GMIME_CRYPTO_PUBKEY_ALGO_RSA_S,
-	GMIME_CRYPTO_PUBKEY_ALGO_ELG,
-	GMIME_CRYPTO_PUBKEY_ALGO_ELG_E,
-	GMIME_CRYPTO_PUBKEY_ALGO_DSA
-} GMimeCryptoPubKeyAlgo;
-
-
 /**
  * GMimeCryptoHash:
  * @GMIME_CRYPTO_HASH_DEFAULT: The default hash algorithm.
- * @GMIME_CRYPTO_HASH_MD2: The MD2 hash algorithm.
  * @GMIME_CRYPTO_HASH_MD5: The MD5 hash algorithm.
  * @GMIME_CRYPTO_HASH_SHA1: The SHA-1 hash algorithm.
- * @GMIME_CRYPTO_HASH_SHA224: The SHA-224 hash algorithm.
+ * @GMIME_CRYPTO_HASH_RIPEMD160: The RIPEMD-160 hash algorithm.
+ * @GMIME_CRYPTO_HASH_MD2: The MD2 hash algorithm.
+ * @GMIME_CRYPTO_HASH_TIGER192: The TIGER-192 hash algorithm.
+ * @GMIME_CRYPTO_HASH_HAVAL5160: The HAVAL-5-160 hash algorithm.
  * @GMIME_CRYPTO_HASH_SHA256: The SHA-256 hash algorithm.
  * @GMIME_CRYPTO_HASH_SHA384: The SHA-384 hash algorithm.
  * @GMIME_CRYPTO_HASH_SHA512: The SHA-512 hash algorithm.
- * @GMIME_CRYPTO_HASH_RIPEMD160: The RIPEMD-160 hash algorithm.
- * @GMIME_CRYPTO_HASH_TIGER192: The TIGER-192 hash algorithm.
- * @GMIME_CRYPTO_HASH_HAVAL5160: The HAVAL-5-160 hash algorithm.
+ * @GMIME_CRYPTO_HASH_SHA224: The SHA-224 hash algorithm.
  * @GMIME_CRYPTO_HASH_MD4: The MD4 hash algorithm.
  *
  * A hash algorithm.
  **/
 typedef enum {
-	GMIME_CRYPTO_HASH_DEFAULT,
-	GMIME_CRYPTO_HASH_MD2,
-	GMIME_CRYPTO_HASH_MD5,
-	GMIME_CRYPTO_HASH_SHA1,
-	GMIME_CRYPTO_HASH_SHA224,
-	GMIME_CRYPTO_HASH_SHA256,
-	GMIME_CRYPTO_HASH_SHA384,
-	GMIME_CRYPTO_HASH_SHA512,
-	GMIME_CRYPTO_HASH_RIPEMD160,
-	GMIME_CRYPTO_HASH_TIGER192,
-	GMIME_CRYPTO_HASH_HAVAL5160,
-	GMIME_CRYPTO_HASH_MD4
+	GMIME_CRYPTO_HASH_DEFAULT     = 0,
+	GMIME_CRYPTO_HASH_MD5         = 1,
+	GMIME_CRYPTO_HASH_SHA1        = 2,
+	GMIME_CRYPTO_HASH_RIPEMD160   = 3,
+	GMIME_CRYPTO_HASH_MD2         = 5,
+	GMIME_CRYPTO_HASH_TIGER192    = 6,
+	GMIME_CRYPTO_HASH_HAVAL5160   = 7,
+	GMIME_CRYPTO_HASH_SHA256      = 8,
+	GMIME_CRYPTO_HASH_SHA384      = 9,
+	GMIME_CRYPTO_HASH_SHA512      = 10,
+	GMIME_CRYPTO_HASH_SHA224      = 11,
+	GMIME_CRYPTO_HASH_MD4         = 301,
 } GMimeCryptoHash;
 
 
@@ -124,9 +102,6 @@ typedef enum {
  * GMimeCryptoContext:
  * @parent_object: parent #GObject
  * @request_passwd: a callback for requesting a password
- * @sign_protocol: signature protocol (must be set by subclass)
- * @encrypt_protocol: encryption protocol (must be set by subclass)
- * @key_protocol: key exchange protocol (must be set by subclass)
  *
  * A crypto context for use with MIME.
  **/
@@ -134,11 +109,6 @@ struct _GMimeCryptoContext {
 	GObject parent_object;
 	
 	GMimePasswordRequestFunc request_passwd;
-	
-	/* these must be set by the subclass in the instance_init() */
-	const char *sign_protocol;
-	const char *encrypt_protocol;
-	const char *key_protocol;
 };
 
 struct _GMimeCryptoContextClass {
@@ -148,6 +118,12 @@ struct _GMimeCryptoContextClass {
 	
 	const char *             (* hash_name)   (GMimeCryptoContext *ctx, GMimeCryptoHash hash);
 	
+	const char *             (* get_signature_protocol) (GMimeCryptoContext *ctx);
+	
+	const char *             (* get_encryption_protocol) (GMimeCryptoContext *ctx);
+	
+	const char *             (* get_key_exchange_protocol) (GMimeCryptoContext *ctx);
+	
 	int                      (* sign)        (GMimeCryptoContext *ctx, const char *userid,
 						  GMimeCryptoHash hash, GMimeStream *istream,
 						  GMimeStream *ostream, GError **err);
@@ -161,7 +137,7 @@ struct _GMimeCryptoContextClass {
 						  GPtrArray *recipients, GMimeStream *istream,
 						  GMimeStream *ostream, GError **err);
 	
-	GMimeSignatureValidity * (* decrypt)     (GMimeCryptoContext *ctx, GMimeStream *istream,
+	GMimeDecryptionResult *  (* decrypt)     (GMimeCryptoContext *ctx, GMimeStream *istream,
 						  GMimeStream *ostream, GError **err);
 	
 	int                      (* import_keys) (GMimeCryptoContext *ctx, GMimeStream *istream,
@@ -177,37 +153,66 @@ GType g_mime_crypto_context_get_type (void);
 void g_mime_crypto_context_set_request_password (GMimeCryptoContext *ctx, GMimePasswordRequestFunc request_passwd);
 
 /* hash routines */
-GMimeCryptoHash      g_mime_crypto_context_hash_id (GMimeCryptoContext *ctx, const char *hash);
+GMimeCryptoHash g_mime_crypto_context_hash_id (GMimeCryptoContext *ctx, const char *hash);
 
-const char *         g_mime_crypto_context_hash_name (GMimeCryptoContext *ctx, GMimeCryptoHash hash);
+const char *g_mime_crypto_context_hash_name (GMimeCryptoContext *ctx, GMimeCryptoHash hash);
+
+/* protocol routines */
+const char *g_mime_crypto_context_get_signature_protocol (GMimeCryptoContext *ctx);
+
+const char *g_mime_crypto_context_get_encryption_protocol (GMimeCryptoContext *ctx);
+
+const char *g_mime_crypto_context_get_key_exchange_protocol (GMimeCryptoContext *ctx);
 
 /* crypto routines */
-int                  g_mime_crypto_context_sign (GMimeCryptoContext *ctx, const char *userid,
-						 GMimeCryptoHash hash, GMimeStream *istream,
-						 GMimeStream *ostream, GError **err);
+int g_mime_crypto_context_sign (GMimeCryptoContext *ctx, const char *userid,
+				GMimeCryptoHash hash, GMimeStream *istream,
+				GMimeStream *ostream, GError **err);
 
 GMimeSignatureValidity *g_mime_crypto_context_verify (GMimeCryptoContext *ctx, GMimeCryptoHash hash,
 						      GMimeStream *istream, GMimeStream *sigstream,
 						      GError **err);
 
-int                  g_mime_crypto_context_encrypt (GMimeCryptoContext *ctx, gboolean sign,
-						    const char *userid, GMimeCryptoHash hash,
-						    GPtrArray *recipients, GMimeStream *istream,
-						    GMimeStream *ostream, GError **err);
+int g_mime_crypto_context_encrypt (GMimeCryptoContext *ctx, gboolean sign,
+				   const char *userid, GMimeCryptoHash hash,
+				   GPtrArray *recipients, GMimeStream *istream,
+				   GMimeStream *ostream, GError **err);
 
-GMimeSignatureValidity *g_mime_crypto_context_decrypt (GMimeCryptoContext *ctx, GMimeStream *istream,
-						       GMimeStream *ostream, GError **err);
+GMimeDecryptionResult *g_mime_crypto_context_decrypt (GMimeCryptoContext *ctx, GMimeStream *istream,
+						      GMimeStream *ostream, GError **err);
 
 /* key/certificate routines */
-int                  g_mime_crypto_context_import_keys (GMimeCryptoContext *ctx, GMimeStream *istream,
-							GError **err);
+int g_mime_crypto_context_import_keys (GMimeCryptoContext *ctx, GMimeStream *istream, GError **err);
 
-int                  g_mime_crypto_context_export_keys (GMimeCryptoContext *ctx, GPtrArray *keys,
-							GMimeStream *ostream, GError **err);
+int g_mime_crypto_context_export_keys (GMimeCryptoContext *ctx, GPtrArray *keys,
+				       GMimeStream *ostream, GError **err);
 
 
 /* signature status structures and functions */
 
+/**
+ * GMimeCryptoPubKeyAlgo:
+ * @GMIME_CRYPTO_PUBKEY_ALGO_DEFAULT: The default public-key algorithm.
+ * @GMIME_CRYPTO_PUBKEY_ALGO_RSA: The RSA algorithm.
+ * @GMIME_CRYPTO_PUBKEY_ALGO_RSA_E: An encryption-only RSA algorithm.
+ * @GMIME_CRYPTO_PUBKEY_ALGO_RSA_S: A signature-only RSA algorithm.
+ * @GMIME_CRYPTO_PUBKEY_ALGO_ELG_E: An encryption-only ElGamal algorithm.
+ * @GMIME_CRYPTO_PUBKEY_ALGO_DSA: The DSA algorithm.
+ * @GMIME_CRYPTO_PUBKEY_ALGO_ELG: The ElGamal algorithm.
+ *
+ * A public-key algorithm.
+ **/
+typedef enum {
+	GMIME_CRYPTO_PUBKEY_ALGO_DEFAULT  = 0,
+	GMIME_CRYPTO_PUBKEY_ALGO_RSA      = 1,
+	GMIME_CRYPTO_PUBKEY_ALGO_RSA_E    = 2,
+	GMIME_CRYPTO_PUBKEY_ALGO_RSA_S    = 3,
+	GMIME_CRYPTO_PUBKEY_ALGO_ELG_E    = 16,
+	GMIME_CRYPTO_PUBKEY_ALGO_DSA      = 17,
+	GMIME_CRYPTO_PUBKEY_ALGO_ELG      = 20
+} GMimeCryptoPubKeyAlgo;
+
+
 /**
  * GMimeSignerTrust:
  * @GMIME_SIGNER_TRUST_NONE: No trust assigned.
@@ -252,15 +257,17 @@ typedef enum {
  * @GMIME_SIGNER_ERROR_NO_PUBKEY: No public key found.
  * @GMIME_SIGNER_ERROR_EXPKEYSIG: Expired signature key.
  * @GMIME_SIGNER_ERROR_REVKEYSIG: Revoked signature key.
+ * @GMIME_SIGNER_ERROR_UNSUPP_ALGO: Unsupported algorithm.
  *
  * Possible errors that a #GMimeSigner could have.
  **/
 typedef enum {
-	GMIME_SIGNER_ERROR_NONE,
+	GMIME_SIGNER_ERROR_NONE        = 0,
 	GMIME_SIGNER_ERROR_EXPSIG      = (1 << 0),  /* expired signature */
 	GMIME_SIGNER_ERROR_NO_PUBKEY   = (1 << 1),  /* no public key */
 	GMIME_SIGNER_ERROR_EXPKEYSIG   = (1 << 2),  /* expired key */
-	GMIME_SIGNER_ERROR_REVKEYSIG   = (1 << 3)   /* revoked key */
+	GMIME_SIGNER_ERROR_REVKEYSIG   = (1 << 3),  /* revoked key */
+	GMIME_SIGNER_ERROR_UNSUPP_ALGO = (1 << 4)   /* unsupported algorithm */
 } GMimeSignerError;
 
 
@@ -271,6 +278,8 @@ typedef enum {
  * @errors: A bitfield of #GMimeSignerError values.
  * @trust: A #GMimeSignerTrust.
  * @unused: Unused expansion bits for future use; ignore this.
+ * @sig_class: Crypto-specific signature class.
+ * @sig_ver: Crypto-specific signature version.
  * @pubkey_algo: The public-key algorithm used by the signer, if known.
  * @hash_algo: The hash algorithm used by the signer, if known.
  * @issuer_serial: The issuer of the certificate, if known.
@@ -289,9 +298,11 @@ typedef enum {
 struct _GMimeSigner {
 	GMimeSigner *next;
 	unsigned int status:2;    /* GMimeSignerStatus */
-	unsigned int errors:4;    /* bitfield of GMimeSignerError's */
+	unsigned int errors:6;    /* bitfield of GMimeSignerError's */
 	unsigned int trust:3;     /* GMimeSignerTrust */
-	unsigned int unused:21;   /* unused expansion bits */
+	unsigned int unused:5;    /* unused expansion bits */
+	unsigned int sig_class:8; /* crypto-specific signature class */
+	unsigned int sig_ver:8;   /* crypto-specific signature version */
 	GMimeCryptoPubKeyAlgo pubkey_algo;
 	GMimeCryptoHash hash_algo;
 	char *issuer_serial;
@@ -308,9 +319,9 @@ struct _GMimeSigner {
 
 
 GMimeSigner *g_mime_signer_new (GMimeSignerStatus status);
-void         g_mime_signer_free (GMimeSigner *signer);
+void g_mime_signer_free (GMimeSigner *signer);
 
-GMimeSigner *g_mime_signer_next (GMimeSigner *signer);
+const GMimeSigner *g_mime_signer_next (const GMimeSigner *signer);
 
 void g_mime_signer_set_status (GMimeSigner *signer, GMimeSignerStatus status);
 GMimeSignerStatus g_mime_signer_get_status (const GMimeSigner *signer);
@@ -321,6 +332,12 @@ GMimeSignerError g_mime_signer_get_errors (const GMimeSigner *signer);
 void g_mime_signer_set_trust (GMimeSigner *signer, GMimeSignerTrust trust);
 GMimeSignerTrust g_mime_signer_get_trust (const GMimeSigner *signer);
 
+void g_mime_signer_set_sig_class (GMimeSigner *signer, int sig_class);
+int g_mime_signer_get_sig_class (const GMimeSigner *signer);
+
+void g_mime_signer_set_sig_version (GMimeSigner *signer, int version);
+int g_mime_signer_get_sig_version (const GMimeSigner *signer);
+
 void g_mime_signer_set_pubkey_algo (GMimeSigner *signer, GMimeCryptoPubKeyAlgo pubkey_algo);
 GMimeCryptoPubKeyAlgo g_mime_signer_get_pubkey_algo (const GMimeSigner *signer);
 
@@ -360,7 +377,6 @@ time_t g_mime_signer_get_key_expires (const GMimeSigner *signer);
 
 /**
  * GMimeSignatureValidity:
- * @status: The overall signature status.
  * @signers: A list of #GMimeSigner structures.
  * @details: A string containing more user-readable details.
  *
@@ -374,13 +390,106 @@ struct _GMimeSignatureValidity {
 
 
 GMimeSignatureValidity *g_mime_signature_validity_new (void);
-void                    g_mime_signature_validity_free (GMimeSignatureValidity *validity);
+void g_mime_signature_validity_free (GMimeSignatureValidity *validity);
 
-const char             *g_mime_signature_validity_get_details (const GMimeSignatureValidity *validity);
-void                    g_mime_signature_validity_set_details (GMimeSignatureValidity *validity, const char *details);
+void g_mime_signature_validity_set_details (GMimeSignatureValidity *validity, const char *details);
+const char *g_mime_signature_validity_get_details (const GMimeSignatureValidity *validity);
 
-const GMimeSigner      *g_mime_signature_validity_get_signers (const GMimeSignatureValidity *validity);
-void                    g_mime_signature_validity_add_signer  (GMimeSignatureValidity *validity, GMimeSigner *signer);
+const GMimeSigner *g_mime_signature_validity_get_signers (const GMimeSignatureValidity *validity);
+void g_mime_signature_validity_add_signer (GMimeSignatureValidity *validity, GMimeSigner *signer);
+
+
+
+/**
+ * GMimeCryptoRecipient:
+ * @next: Pointer to the next #GMimeCryptoRecipient.
+ * @pubkey_algo: The public-key algorithm used by the recipient, if known.
+ * @keyid: The recipient's key id.
+ *
+ * A structure containing useful information about a recipient.
+ **/
+struct _GMimeCryptoRecipient {
+	GMimeCryptoRecipient *next;
+	GMimeCryptoPubKeyAlgo pubkey_algo;
+	char *keyid;
+};
+
+
+GMimeCryptoRecipient *g_mime_crypto_recipient_new (void);
+void g_mime_crypto_recipient_free (GMimeCryptoRecipient *recipient);
+
+const GMimeCryptoRecipient *g_mime_crypto_recipient_next (const GMimeCryptoRecipient *recipient);
+
+void g_mime_crypto_recipient_set_pubkey_algo (GMimeCryptoRecipient *recipient, GMimeCryptoPubKeyAlgo pubkey_algo);
+GMimeCryptoPubKeyAlgo g_mime_crypto_recipient_get_pubkey_algo (const GMimeCryptoRecipient *recipient);
+
+void g_mime_crypto_recipient_set_key_id (GMimeCryptoRecipient *recipient, const char *key_id);
+const char *g_mime_crypto_recipient_get_key_id (const GMimeCryptoRecipient *recipient);
+
+
+/**
+ * GMimeCryptoCipherAlgo:
+ * @GMIME_CRYPTO_CIPHER_ALGO_DEFAULT: The default (or unknown) cipher.
+ * @GMIME_CRYPTO_CIPHER_ALGO_IDEA: The IDEA cipher.
+ * @GMIME_CRYPTO_CIPHER_ALGO_3DES: The 3DES cipher.
+ * @GMIME_CRYPTO_CIPHER_ALGO_CAST5: The CAST5 cipher.
+ * @GMIME_CRYPTO_CIPHER_ALGO_BLOWFISH: The Blowfish cipher.
+ * @GMIME_CRYPTO_CIPHER_ALGO_AES: The AES (aka RIJANDALE) cipher.
+ * @GMIME_CRYPTO_CIPHER_ALGO_AES192: The AES-192 cipher.
+ * @GMIME_CRYPTO_CIPHER_ALGO_AES256: The AES-256 cipher.
+ * @GMIME_CRYPTO_CIPHER_ALGO_TWOFISH: The Twofish cipher.
+ * @GMIME_CRYPTO_CIPHER_ALGO_CAMELLIA128: The Camellia-128 cipher.
+ * @GMIME_CRYPTO_CIPHER_ALGO_CAMELLIA196: The Camellia-196 cipher.
+ * @GMIME_CRYPTO_CIPHER_ALGO_CAMELLIA256: The Camellia-256 cipher.
+ *
+ * A cipher algorithm.
+ **/
+typedef enum {
+	GMIME_CRYPTO_CIPHER_ALGO_DEFAULT     = 0,
+	GMIME_CRYPTO_CIPHER_ALGO_IDEA        = 1,
+	GMIME_CRYPTO_CIPHER_ALGO_3DES        = 2,
+	GMIME_CRYPTO_CIPHER_ALGO_CAST5       = 3,
+	GMIME_CRYPTO_CIPHER_ALGO_BLOWFISH    = 4,
+	GMIME_CRYPTO_CIPHER_ALGO_AES         = 7,
+	GMIME_CRYPTO_CIPHER_ALGO_AES192      = 8,
+	GMIME_CRYPTO_CIPHER_ALGO_AES256      = 9,
+	GMIME_CRYPTO_CIPHER_ALGO_TWOFISH     = 10,
+	GMIME_CRYPTO_CIPHER_ALGO_CAMELLIA128 = 11,
+	GMIME_CRYPTO_CIPHER_ALGO_CAMELLIA196 = 12,
+	GMIME_CRYPTO_CIPHER_ALGO_CAMELLIA256 = 13
+} GMimeCryptoCipherAlgo;
+
+/**
+ * GMimeDecryptionResult:
+ * @validity: A #GMimeSignatureValidity if signed or %NULL otherwise.
+ * @recipients: A list of #GMimeCryptoRecipient structures.
+ * @cipher: The cipher algorithm used to encrypt the stream.
+ * @mdc: The MDC digest algorithm used, if any.
+ *
+ * A structure containing the results from decrypting an encrypted stream.
+ **/
+struct _GMimeDecryptionResult {
+	GMimeSignatureValidity *validity;
+	GMimeCryptoRecipient *recipients;
+	GMimeCryptoCipherAlgo cipher;
+	GMimeCryptoHash mdc;
+};
+
+
+GMimeDecryptionResult *g_mime_decryption_result_new (void);
+void g_mime_decryption_result_free (GMimeDecryptionResult *result);
+
+void g_mime_decryption_result_set_validity (GMimeDecryptionResult *result, GMimeSignatureValidity *validity);
+const GMimeSignatureValidity *g_mime_decryption_result_get_validity (const GMimeDecryptionResult *result);
+
+void g_mime_decryption_result_add_recipient (GMimeDecryptionResult *result, GMimeCryptoRecipient *recipient);
+const GMimeCryptoRecipient *g_mime_decryption_result_get_recipients (const GMimeDecryptionResult *result);
+
+void g_mime_decryption_result_set_cipher (GMimeDecryptionResult *result, GMimeCryptoCipherAlgo cipher);
+GMimeCryptoCipherAlgo g_mime_decryption_result_get_cipher (const GMimeDecryptionResult *result);
+
+void g_mime_decryption_result_set_mdc (GMimeDecryptionResult *result, GMimeCryptoHash mdc);
+GMimeCryptoHash g_mime_decryption_result_get_mdc (const GMimeDecryptionResult *result);
 
 G_END_DECLS
 
