@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2010 Jeffrey Stedfast
+ *  Copyright (C) 2000-2011 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -71,7 +71,7 @@ test_sign (GMimeCryptoContext *ctx, GMimeStream *cleartext, GMimeStream *ciphert
 	Exception *ex;
 	int rv;
 	
-	rv = g_mime_crypto_context_sign (ctx, "no.user@no.domain",
+	rv = g_mime_crypto_context_sign (ctx, "alice@example.net",
 					 GMIME_CRYPTO_HASH_SHA256,
 					 cleartext, ciphertext, &err);
 	
@@ -119,9 +119,9 @@ test_encrypt (GMimeCryptoContext *ctx, gboolean sign, GMimeStream *cleartext, GM
 	Exception *ex;
 	
 	recipients = g_ptr_array_new ();
-	g_ptr_array_add (recipients, "no.user@no.domain");
+	g_ptr_array_add (recipients, "alice@example.net");
 	
-	g_mime_crypto_context_encrypt (ctx, sign, "no.user@no.domain",
+	g_mime_crypto_context_encrypt (ctx, sign, "alice@example.net",
 				       GMIME_CRYPTO_HASH_SHA256, recipients,
 				       cleartext, ciphertext, &err);
 	
@@ -141,7 +141,7 @@ test_encrypt (GMimeCryptoContext *ctx, gboolean sign, GMimeStream *cleartext, GM
 static void
 test_decrypt (GMimeCryptoContext *ctx, gboolean sign, GMimeStream *cleartext, GMimeStream *ciphertext)
 {
-	GMimeSignatureValidity *sv;
+	GMimeDecryptionResult *result;
 	Exception *ex = NULL;
 	GMimeStream *stream;
 	GError *err = NULL;
@@ -149,7 +149,7 @@ test_decrypt (GMimeCryptoContext *ctx, gboolean sign, GMimeStream *cleartext, GM
 	
 	stream = g_mime_stream_mem_new ();
 	
-	if (!(sv = g_mime_crypto_context_decrypt (ctx, ciphertext, stream, &err))) {
+	if (!(result = g_mime_crypto_context_decrypt (ctx, ciphertext, stream, &err))) {
 		g_object_unref (stream);
 		ex = exception_new ("%s", err->message);
 		g_error_free (err);
@@ -157,14 +157,14 @@ test_decrypt (GMimeCryptoContext *ctx, gboolean sign, GMimeStream *cleartext, GM
 	}
 	
 	if (sign) {
-		if (get_sig_status (sv->signers) != GMIME_SIGNER_STATUS_GOOD)
+		if (!result->validity || get_sig_status (result->validity->signers) != GMIME_SIGNER_STATUS_GOOD)
 			ex = exception_new ("expected GOOD signature");
 	} else {
-		if (sv->signers != NULL)
+		if (result->validity && result->validity->signers != NULL)
 			ex = exception_new ("unexpected signature");
 	}
 	
-	g_mime_signature_validity_free (sv);
+	g_mime_decryption_result_free (result);
 	
 	if (ex != NULL) {
 		g_object_unref (stream);
@@ -205,7 +205,7 @@ test_export (GMimeCryptoContext *ctx, const char *path)
 	g_object_unref (ostream);
 	
 	keys = g_ptr_array_new ();
-	g_ptr_array_add (keys, "no.user@no.domain");
+	g_ptr_array_add (keys, "alice@example.net");
 	
 	ostream = g_mime_stream_mem_new ();
 	
@@ -263,7 +263,7 @@ import_key (GMimeCryptoContext *ctx, const char *path)
 	g_object_unref (stream);
 	
 	if (err != NULL) {
-		ex = exception_new ("%s", err->message);
+		ex = exception_new ("%s: %s", err->message, gpg_strerror (err->code));
 		g_error_free (err);
 		throw (ex);
 	}
@@ -306,11 +306,13 @@ int main (int argc, char **argv)
 	
 	testsuite_check ("GMimePkcs7Context::import");
 	try {
-		key = g_build_filename (datadir, "gmime-cert.p7", NULL);
+		key = g_build_filename (datadir, "alice.pem", NULL);
+		//printf ("importing key: %s\n", key);
 		import_key (ctx, key);
 		g_free (key);
 		
-		key = g_build_filename (datadir, "gmime-cert.p12", NULL);
+		key = g_build_filename (datadir, "alice.p12", NULL);
+		//printf ("importing key: %s\n", key);
 		import_key (ctx, key);
 		g_free (key);
 		
@@ -320,7 +322,7 @@ int main (int argc, char **argv)
 		return EXIT_FAILURE;
 	} finally;
 	
-	key = g_build_filename (datadir, "gmime-cert.p7", NULL);
+	key = g_build_filename (datadir, "alice.pem", NULL);
 	testsuite_check ("GMimePkcs7Context::export");
 	try {
 		test_export (ctx, key);
